@@ -1,6 +1,8 @@
 // -*- mode: C++; c-file-style: "cc-mode" -*-
 //*************************************************************************
 //
+// Code available from: https://verilator.org
+//
 // Copyright 2009-2021 by Wilson Snyder. This program is free software; you can
 // redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
@@ -10,14 +12,15 @@
 //=========================================================================
 ///
 /// \file
-/// \brief Verilator: Implementation Header, only for verilated.cpp internals.
+/// \brief Verilated implementation Header, only for verilated.cpp internals.
 ///
-/// Code available from: https://verilator.org
+/// This file is not part of the Verilated public-facing API.
+/// It is only for internal use by the Verilated libraries.
 ///
 //=========================================================================
 
 #ifndef VERILATOR_VERILATED_IMP_H_
-#define VERILATOR_VERILATED_IMP_H_  ///< Header Guard
+#define VERILATOR_VERILATED_IMP_H_
 
 // clang-format off
 #if !defined(VERILATOR_VERILATED_CPP_) && !defined(VERILATOR_VERILATED_DPI_CPP_) \
@@ -46,7 +49,7 @@ class VerilatedScope;
 // Threaded message passing
 
 #ifdef VL_THREADED
-/// Message, enqueued on an mtask, and consumed on the main eval thread
+// Message, enqueued on an mtask, and consumed on the main eval thread
 class VerilatedMsg final {
 public:
     // TYPES
@@ -58,8 +61,8 @@ public:
 
 private:
     // MEMBERS
-    vluint32_t m_mtaskId;  ///< MTask that did enqueue
-    std::function<void()> m_cb;  ///< Lambda to execute when message received
+    vluint32_t m_mtaskId;  // MTask that did enqueue
+    std::function<void()> m_cb;  // Lambda to execute when message received
 public:
     // CONSTRUCTORS
     explicit VerilatedMsg(const std::function<void()>& cb)
@@ -72,20 +75,20 @@ public:
     VerilatedMsg& operator=(VerilatedMsg&&) = default;
     // METHODS
     vluint32_t mtaskId() const { return m_mtaskId; }
-    /// Execute the lambda function
+    // Execute the lambda function
     void run() const { m_cb(); }
 };
 
-/// Each thread has a queue it pushes to
-/// This assumes no thread starts pushing the next tick until the previous has drained.
-/// If more aggressiveness is needed, a double-buffered scheme might work well.
+// Each thread has a queue it pushes to
+// This assumes no thread starts pushing the next tick until the previous has drained.
+// If more aggressiveness is needed, a double-buffered scheme might work well.
 class VerilatedEvalMsgQueue final {
-    typedef std::multiset<VerilatedMsg, VerilatedMsg::Cmp> VerilatedThreadQueue;
+    using VerilatedThreadQueue = std::multiset<VerilatedMsg, VerilatedMsg::Cmp>;
 
-    std::atomic<vluint64_t> m_depth;  ///< Current depth of queue (see comments below)
+    std::atomic<vluint64_t> m_depth;  // Current depth of queue (see comments below)
 
-    VerilatedMutex m_mutex;  ///< Mutex protecting queue
-    VerilatedThreadQueue m_queue VL_GUARDED_BY(m_mutex);  ///< Message queue
+    VerilatedMutex m_mutex;  // Mutex protecting queue
+    VerilatedThreadQueue m_queue VL_GUARDED_BY(m_mutex);  // Message queue
 public:
     // CONSTRUCTORS
     VerilatedEvalMsgQueue()
@@ -99,13 +102,13 @@ private:
 
 public:
     // METHODS
-    //// Add message to queue (called by producer)
+    // Add message to queue (called by producer)
     void post(const VerilatedMsg& msg) VL_MT_SAFE_EXCLUDES(m_mutex) {
         const VerilatedLockGuard lock(m_mutex);
         m_queue.insert(msg);  // Pass by value to copy the message into queue
         ++m_depth;
     }
-    /// Service queue until completion (called by consumer)
+    // Service queue until completion (called by consumer)
     void process() VL_MT_SAFE_EXCLUDES(m_mutex) {
         // Tracking m_depth is redundant to e.g. getting the mutex and looking at queue size,
         // but on the reader side it's 4x faster to test an atomic then getting a mutex
@@ -130,7 +133,7 @@ public:
     }
 };
 
-/// Each thread has a local queue to build up messages until the end of the eval() call
+// Each thread has a local queue to build up messages until the end of the eval() call
 class VerilatedThreadMsgQueue final {
     std::queue<VerilatedMsg> m_queue;
 
@@ -151,7 +154,7 @@ private:
     }
 
 public:
-    /// Add message to queue, called by producer
+    // Add message to queue, called by producer
     static void post(const VerilatedMsg& msg) VL_MT_SAFE {
         // Handle calls to threaded routines outside
         // of any mtask -- if an initial block calls $finish, say.
@@ -163,7 +166,7 @@ public:
             threadton().m_queue.push(msg);  // Pass by value to copy the message into queue
         }
     }
-    /// Push all messages to the eval's queue
+    // Push all messages to the eval's queue
     static void flush(VerilatedEvalMsgQueue* evalMsgQp) VL_MT_SAFE {
         while (!threadton().m_queue.empty()) {
             evalMsgQp->post(threadton().m_queue.front());
@@ -180,7 +183,7 @@ class VerilatedFpList final {
     std::size_t m_sz = 0;
 
 public:
-    typedef FILE* const* const_iterator;
+    using const_iterator = FILE* const*;
     explicit VerilatedFpList() {}
     const_iterator begin() const { return m_fp; }
     const_iterator end() const { return m_fp + m_sz; }
@@ -194,16 +197,16 @@ public:
 //======================================================================
 // VerilatedContextImpData
 
-/// Class for hidden implementation members inside VerilatedContext
+// Class for hidden implementation members inside VerilatedContext
 // Avoids needing std::unordered_map inside verilated.h
 class VerilatedContextImpData final {
     friend class VerilatedContext;
     friend class VerilatedContextImp;
 
 protected:
-    /// Map of <scope_name, scope pointer>
+    // Map of <scope_name, scope pointer>
     // Used by scopeInsert, scopeFind, scopeErase, scopeNameMap
-    mutable VerilatedMutex m_nameMutex;  ///< Protect m_nameMap
+    mutable VerilatedMutex m_nameMutex;  // Protect m_nameMap
     VerilatedScopeNameMap m_nameMap VL_GUARDED_BY(m_nameMutex);
 };
 
@@ -242,7 +245,7 @@ public:  // But only for verilated*.cpp
     static vluint32_t randSeedEpoch() VL_MT_SAFE { return s().s_randSeedEpoch; }
 
     // METHODS - timeformat
-    int timeFormatUnits() VL_MT_SAFE {
+    int timeFormatUnits() const VL_MT_SAFE {
         if (m_s.m_timeFormatUnits == VerilatedContext::Serialized::UNITS_NONE)
             return timeprecision();
         return m_s.m_timeFormatUnits;
@@ -252,7 +255,7 @@ public:  // But only for verilated*.cpp
     void timeFormatPrecision(int value) VL_MT_SAFE { m_s.m_timeFormatPrecision = value; }
     int timeFormatWidth() const VL_MT_SAFE { return m_s.m_timeFormatWidth; }
     void timeFormatWidth(int value) VL_MT_SAFE { m_s.m_timeFormatWidth = value; }
-    std::string timeFormatSuffix() VL_MT_SAFE_EXCLUDES(m_timeDumpMutex) {
+    std::string timeFormatSuffix() const VL_MT_SAFE_EXCLUDES(m_timeDumpMutex) {
         const VerilatedLockGuard lock(m_timeDumpMutex);
         return m_timeFormatSuffix;
     }
@@ -277,12 +280,12 @@ public:  // But only for verilated*.cpp
         if (m_fdFreeMct.empty()) return 0;
         IData idx = m_fdFreeMct.back();
         m_fdFreeMct.pop_back();
-        m_fdps[idx] = fopen(filenamep, "w");
+        m_fdps[idx] = std::fopen(filenamep, "w");
         if (VL_UNLIKELY(!m_fdps[idx])) return 0;
         return (1 << idx);
     }
     IData fdNew(const char* filenamep, const char* modep) VL_MT_SAFE_EXCLUDES(m_fdMutex) {
-        FILE* fp = fopen(filenamep, modep);
+        FILE* fp = std::fopen(filenamep, modep);
         if (VL_UNLIKELY(!fp)) return 0;
         // Bit 31 indicates it's a descriptor not a MCD
         const VerilatedLockGuard lock(m_fdMutex);
@@ -305,20 +308,20 @@ public:  // But only for verilated*.cpp
     void fdFlush(IData fdi) VL_MT_SAFE_EXCLUDES(m_fdMutex) {
         const VerilatedLockGuard lock(m_fdMutex);
         const VerilatedFpList fdlist = fdToFpList(fdi);
-        for (const auto& i : fdlist) fflush(i);
+        for (const auto& i : fdlist) std::fflush(i);
     }
     IData fdSeek(IData fdi, IData offset, IData origin) VL_MT_SAFE_EXCLUDES(m_fdMutex) {
         const VerilatedLockGuard lock(m_fdMutex);
         const VerilatedFpList fdlist = fdToFpList(fdi);
         if (VL_UNLIKELY(fdlist.size() != 1)) return 0;
         return static_cast<IData>(
-            fseek(*fdlist.begin(), static_cast<long>(offset), static_cast<int>(origin)));
+            std::fseek(*fdlist.begin(), static_cast<long>(offset), static_cast<int>(origin)));
     }
     IData fdTell(IData fdi) VL_MT_SAFE_EXCLUDES(m_fdMutex) {
         const VerilatedLockGuard lock(m_fdMutex);
         const VerilatedFpList fdlist = fdToFpList(fdi);
         if (VL_UNLIKELY(fdlist.size() != 1)) return 0;
-        return static_cast<IData>(ftell(*fdlist.begin()));
+        return static_cast<IData>(std::ftell(*fdlist.begin()));
     }
     void fdWrite(IData fdi, const std::string& output) VL_MT_SAFE_EXCLUDES(m_fdMutex) {
         const VerilatedLockGuard lock(m_fdMutex);
@@ -330,19 +333,22 @@ public:  // But only for verilated*.cpp
     }
     void fdClose(IData fdi) VL_MT_SAFE_EXCLUDES(m_fdMutex) {
         const VerilatedLockGuard lock(m_fdMutex);
-        if ((fdi & (1 << 31)) != 0) {
+        if (VL_BITISSET_I(fdi, 31)) {
             // Non-MCD case
             IData idx = VL_MASK_I(31) & fdi;
             if (VL_UNLIKELY(idx >= m_fdps.size())) return;
+            if (VL_UNLIKELY(idx <= 2)) return;  // stdout/stdin/stderr
             if (VL_UNLIKELY(!m_fdps[idx])) return;  // Already free
-            fclose(m_fdps[idx]);
+            std::fclose(m_fdps[idx]);
             m_fdps[idx] = (FILE*)0;
             m_fdFree.push_back(idx);
         } else {
             // MCD case
-            for (int i = 0; (fdi != 0) && (i < 31); i++, fdi >>= 1) {
+            // Starts at 1 to skip stdout
+            fdi >>= 1;
+            for (int i = 1; (fdi != 0) && (i < 31); i++, fdi >>= 1) {
                 if (fdi & VL_MASK_I(1)) {
-                    fclose(m_fdps[i]);
+                    std::fclose(m_fdps[i]);
                     m_fdps[i] = nullptr;
                     m_fdFreeMct.push_back(i);
                 }
@@ -372,7 +378,9 @@ private:
             }
         } else {
             // MCD Case
-            for (size_t i = 0; (fdi != 0) && (i < fp.capacity()); ++i, fdi >>= 1) {
+            if (fdi & 1) fp.push_back(stdout);
+            fdi >>= 1;
+            for (size_t i = 1; (fdi != 0) && (i < fp.capacity()); ++i, fdi >>= 1) {
                 if (fdi & VL_MASK_I(1)) fp.push_back(m_fdps[i]);
             }
         }
@@ -398,28 +406,28 @@ protected:
     friend class VerilatedImp;
 
     // TYPES
-    typedef std::map<std::pair<const void*, void*>, void*> UserMap;
-    typedef std::map<const char*, int, VerilatedCStrCmp> ExportNameMap;
+    using UserMap = std::map<std::pair<const void*, void*>, void*>;
+    using ExportNameMap = std::map<const char*, int, VerilatedCStrCmp>;
 
     // MEMBERS
     // Nothing below here is save-restored; users expected to re-register appropriately
 
-    VerilatedMutex m_userMapMutex;  ///< Protect m_userMap
+    VerilatedMutex m_userMapMutex;  // Protect m_userMap
     // For userInsert, userFind.  As indexed by pointer is common across contexts.
-    UserMap m_userMap VL_GUARDED_BY(m_userMapMutex);  ///< Map of <(scope,userkey), userData>
+    UserMap m_userMap VL_GUARDED_BY(m_userMapMutex);  // Map of <(scope,userkey), userData>
 
-    VerilatedMutex m_hierMapMutex;  ///< Protect m_hierMap
-    /// Map that represents scope hierarchy
+    VerilatedMutex m_hierMapMutex;  // Protect m_hierMap
+    // Map that represents scope hierarchy
     // Used by hierarchyAdd, hierarchyRemove, hierarchyMap
     VerilatedHierarchyMap m_hierMap VL_GUARDED_BY(m_hierMapMutex);
 
     // Slow - somewhat static:
-    VerilatedMutex m_exportMutex;  ///< Protect m_nameMap
-    /// Map of <export_func_proto, func number>
+    VerilatedMutex m_exportMutex;  // Protect m_nameMap
+    // Map of <export_func_proto, func number>
     // Used by exportInsert, exportFind, exportName.
     // Export numbers same across all contexts as just a string-to-number conversion
     ExportNameMap m_exportMap VL_GUARDED_BY(m_exportMutex);
-    int m_exportNext VL_GUARDED_BY(m_exportMutex) = 0;  ///< Next export funcnum
+    int m_exportNext VL_GUARDED_BY(m_exportMutex) = 0;  // Next export funcnum
 
     // CONSTRUCTORS
     VerilatedImpData() = default;
@@ -471,7 +479,7 @@ public:
     }
 
 public:  // But only for verilated.cpp
-    /// Symbol table destruction cleans up the entries for each scope.
+    // Symbol table destruction cleans up the entries for each scope.
     static void userEraseScope(const VerilatedScope* scopep) VL_MT_SAFE {
         // Slow ok - called once/scope on destruction, so we simply iterate.
         const VerilatedLockGuard lock(s().m_userMapMutex);
