@@ -1,6 +1,8 @@
 // -*- mode: C++; c-file-style: "cc-mode" -*-
 //*************************************************************************
 //
+// Code available from: https://verilator.org
+//
 // Copyright 2003-2021 by Wilson Snyder. This program is free software; you can
 // redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
@@ -10,17 +12,19 @@
 //*************************************************************************
 ///
 /// \file
-/// \brief Verilator: Common include for target specific intrinsics.
+/// \brief Verilator common target specific intrinsics header
 ///
-///     Code using machine specific intrinsics for optimization should
-///     include this header rather than directly including he target
-///     specific headers. We provide macros to check for availability
-///     of instruction sets, and a common mechanism to disable them.
+/// This file is not part of the Verilated public-facing API.
+///
+/// It is only for internal use; code using machine specific intrinsics for
+/// optimization should include this header rather than directly including
+/// he target specific headers. We provide macros to check for availability
+/// of instruction sets, and a common mechanism to disable them.
 ///
 //*************************************************************************
 
-#ifndef _VERILATED_INTRINSICS_H_
-#define _VERILATED_INTRINSICS_H_ 1  ///< Header Guard
+#ifndef VERILATOR_VERILATED_INTRINSICS_H_
+#define VERILATOR_VERILATED_INTRINSICS_H_
 
 // clang-format off
 
